@@ -1,3 +1,9 @@
+"""
+packing.py
+------------
+
+Pack multiple 2D regions onto larger 2D regions.
+"""
 import time
 import numpy as np
 
@@ -28,8 +34,8 @@ class RectangleBin:
 
         # bounds: (minx, miny, maxx, maxy)
         if bounds is not None:
-            self.bounds = np.asanyarray(bounds,
-                                        dtype=np.float64)
+            self.bounds = np.asanyarray(
+                bounds, dtype=np.float64)
         elif size is not None:
             self.bounds = np.append(
                 [0.0, 0.0], size).astype(np.float64)
@@ -174,9 +180,10 @@ def rectangles_single(rectangles, sheet_size=None, shuffle=False):
                       rectangles[:, 1].max() * 2]
 
     if shuffle:
-        shuffle_len = int(np.random.random() * len(rectangles)) - 1
-        box_order[0:shuffle_len] = np.random.permutation(
-            box_order[0:shuffle_len])
+        # maximum index to shuffle
+        max_idx = int(np.random.random() * len(rectangles)) - 1
+        # reorder with permutations
+        box_order[:max_idx] = np.random.permutation(box_order[:max_idx])
 
     # start the tree
     sheet = RectangleBin(size=sheet_size)
@@ -240,7 +247,8 @@ def polygons(polygons,
              iterations=50,
              density_escape=.95,
              spacing=0.094,
-             quantity=None):
+             quantity=None,
+             **kwargs):
     """
     Pack polygons into a rectangle by taking each Polygon's OBB
     and then packing that as a rectangle.
@@ -285,7 +293,7 @@ def polygons(polygons,
     # move the OBB transform so the polygon is centered
     # in the padded rectangle
     for i, r in enumerate(rect):
-        obb[i][0:2, 2] += r * .5
+        obb[i][:2, 2] += r * .5
 
     # for polygons occurring multiple times
     indexes = np.hstack([np.ones(q, dtype=np.int64) * i
@@ -298,11 +306,6 @@ def polygons(polygons,
     tic = time.time()
     density = 0.0
 
-    # if no sheet size specified, make a large one
-    if sheet_size is None:
-        sheet_size = [rect[:, 0].sum(),
-                      rect[:, 1].max() * 2]
-
     # run packing for a number of iterations
     (density,
      offset,
@@ -312,7 +315,7 @@ def polygons(polygons,
          sheet_size=sheet_size,
          spacing=spacing,
          density_escape=density_escape,
-         iterations=iterations)
+         iterations=iterations, **kwargs)
 
     toc = time.time()
     log.debug('packing finished %i iterations in %f seconds',
@@ -334,7 +337,8 @@ def rectangles(rectangles,
                sheet_size=None,
                density_escape=0.9,
                spacing=0.0,
-               iterations=50):
+               iterations=50,
+               quanta=None):
     """
     Run multiple iterations of rectangle packing.
 
@@ -350,6 +354,8 @@ def rectangles(rectangles,
       Distance to allow between rectangles
     iterations : int
       Number of iterations to run
+    quanta : None or float
+
 
     Returns
     ---------
@@ -368,10 +374,6 @@ def rectangles(rectangles,
     best_density = 0.0
     # how many rectangles were inserted
     best_insert = 0
-    # if no sheet size specified, make a large one
-    if sheet_size is None:
-        sheet_size = [rectangles[:, 0].sum(),
-                      rectangles[:, 1].max() * 2]
 
     for i in range(iterations):
         # run a single insertion order
@@ -383,6 +385,12 @@ def rectangles(rectangles,
         density = packed[0]
         insert = packed[2].sum()
 
+        if quanta is not None:
+            # compute the density using an upsized quanta
+            box = np.ceil(packed[3] / quanta) * quanta
+            # scale the density result
+            density *= (np.product(packed[3]) / np.product(box))
+
         # compare this packing density against our best
         if density > best_density or insert > best_insert:
             best_density = density
@@ -432,6 +440,7 @@ def images(images, power_resize=False):
 
     size = sheet.round().astype(int)
     if power_resize:
+        # round up all dimensions to powers of 2
         size = (2 ** np.ceil(np.log2(size))).astype(np.int64)
 
     # create the image
